import { bdtowgs84, wgs84tobd } from './gps';

const BMap = window.BMap;
const BMapLib = window.BMapLib;

class NeBMap {
  constructor(dom, opt) {
    const define = {
      zoom: 17,
      center: new BMap.Point(116.397428, 39.90923),
      toolbar: false,
    };
    const center = opt.center;
    if (center) {
      opt.center = new BMap.Point(center[0], center[1]);
    }
    const option = Object.assign({}, define, opt);
    this.map = new BMap.Map(dom);
    this.map.centerAndZoom(option.center, option.zoom);
    this.map.enableScrollWheelZoom(true);

    if (option.toolbar) {
      this.map.addControl(new BMap.NavigationControl());
      this.map.addControl(new BMap.ScaleControl());
    }
  }

  setMap(opt) {
    const define = {
      zoom: this.map.getZoom(),
      center: this.map.getCenter(),
    };
    const center = opt.center;
    if (center) {
      opt.center = new BMap.Point(center[0], center[1]);
    }
    const option = Object.assign(define, opt);
    this.map.centerAndZoom(option.center, option.zoom);
  }

  addMarker(pos, opt, cb) {
    const define = {
      image: 'https://webapi.amap.com/theme/v1.3/markers/n/mark_b.png',
      size: new BMap.Size(19, 33),
    };
    const size = opt.size;
    if (size) {
      opt.size = new BMap.Size(size[0], size[1]);
    }
    if (opt.anchor) {
      opt.anchor = new BMap.Size(opt.anchor[0], opt.anchor[1]);
    } else {
      opt.anchor = new BMap.Size(size[0] / 2, size[1]);
    }
    const option = Object.assign({}, define, opt);
    const myIcon = new BMap.Icon(option.image, option.size, {
      anchor: opt.anchor,
    });
    const markerObj = { icon: myIcon };
    if (option.rotation) {
      markerObj.rotation = option.rotation;
    }
    const marker = new BMap.Marker(new BMap.Point(pos[0], pos[1]), markerObj);
    this.map.addOverlay(marker);
    if (cb) {
      marker.addEventListener('click', cb);
    }
    return marker;
  }

  markerMove(marker, target, moveSpeed) {
    const targetPos = new BMap.Point(target[0], target[1]);
    const initPos = marker.getPosition();
    /* eslint-disable */
    let i = 0;
    function resetMkPoint(i) {
      const lng = (targetPos.lng - initPos.lng) * i * moveSpeed / 1000000 + initPos.lng;
      const lat = (targetPos.lat - initPos.lat) * i * moveSpeed / 1000000 + initPos.lat;
      marker.setPosition(new BMap.Point(lng, lat));
      if (i < 1000000 / moveSpeed) {
        setTimeout(() => {
          i++;
          resetMkPoint(i);
        }, 1/moveSpeed);
      }
    }
    resetMkPoint(1);
  }

  clearMap() {
    this.map.clearOverlays();
  }

  openInfoWindow(pos, info) {
    const infoWindow = new BMap.InfoWindow(info);
    this.map.openInfoWindow(infoWindow, new BMap.Point(pos[0], pos[1]));
  }

  addHeatMap(points, opt) {
    const define = {
      radius: 25,
      opacity: [ 0, 0.8 ],
      gradient: {
        0.4: 'blue',
        0.5: 'rgb(117,211,248)',
        0.6: 'rgb(0, 255, 0)',
        0.7: '#ffea00',
        0.8: 'red',
      },
    };
    const option = Object.assign({}, define, opt);
    const heatmap = new BMapLib.HeatmapOverlay(option);
    console.log(heatmap);
    this.heatmap = heatmap;
    if (heatmap) {
      this.map.addOverlay(heatmap);
      heatmap.setDataSet({ data: points, max: 100 });
    }
  }

  showHeatMap() {
    this.heatmap.show();
  }

  hideHeatMap() {
    this.heatmap.hide();
  }

  addMoveLine(lineArr, moveSpeed, lineOpt, markerOpt) {
    const pathArr = [];
    for (const item of lineArr) {
      pathArr.push(new BMap.Point(item[0], item[1]));
    }
    const defineLine = {
      strokeColor: '#00A',
      strokeWeight: 3,
    };
    const lineOption = Object.assign({}, defineLine, lineOpt);
    const line = new BMap.Polyline(pathArr, lineOption);
    this.map.addOverlay(line);
    this.map.setViewport(pathArr);
    const defineMarker = {
      image: 'https://webapi.amap.com/images/car.png',
      size: new BMap.Size(52, 26),
    };
    const size = markerOpt.size;
    if (size) {
      markerOpt.size = new BMap.Size(size[0], size[1]);
    }
    const markerOption = Object.assign({}, defineMarker, markerOpt);
    const lushu = new BMapLib.LuShu(this.map, pathArr, {
      autoView: true,
      icon: new BMap.Icon(markerOption.image, markerOption.size),
      speed: moveSpeed,
      enableRotation: true,
      landmarkPois: [],
    });
    this.lushu = lushu;
  }

  startMove() {
    this.lushu.stop();
    this.lushu.start();
  }

  pauseMove() {
    this.lushu.pause();
  }

  resumeMove() {
    this.lushu.start();
  }

  stopMove() {
    this.lushu.stop();
  }

  static convert(gps) {
    const latlng = [];
    for (const item of gps) {
      latlng.push(new BMap.Point(item[0], item[1]));
    }
    return new Promise(async resolve => {
      const len = gps.length;
      const rest = len % 10;
      let count = len / 10;
      count = rest ? parseInt(count) + 1 : count;
      const location = [];
      for (let i = 0; i < count; i++) {
        const num = i === count ? rest : 10;
        const promise = new Promise((resolve, reject) => {
          new BMap.Convertor().translate(latlng.slice(i * num, (i + 1) * num), 1, 5, result => {
            if (result.status === 0) {
              resolve(result);
            } else {
              reject(result);
            }
          });
        });
        await promise.then(result => {
          for (const item of result.points) {
            location.push([ item.lng, item.lat ]);
          }
        }).catch(err => {
          console.log(err);
        });
      }
      resolve(location);
    });
  }
  translateFromGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      const trans = wgs84tobd(data[j].latitude, data[j].longitude);
      transRes.push(new BMap.Point(trans[0], trans[1]));
    }
    return transRes;
  }
  translateToGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      let trans = [];
      if (data[j].lat) {
        trans = bdtowgs84(data[j].lat, data[j].lng);
      } else {
        trans = bdtowgs84(data[j].latitude, data[j].longitude);
      }
      transRes.push({ latitude: trans[0], longitude: trans[1] });
    }
    return transRes;
  }
  static geocode(location) {
    const point = new BMap.Point(location[0], location[1]);
    return new Promise((resolve, reject) => {
      new BMap.Geocoder().getLocation(point, result => {
        if (result) {
          resolve(result.address);
        } else {
          reject(result);
        }
      });
    });
  }
}

export default NeBMap;
