const qq = window.qq;
const QQMapPlugin = window.QQMapPlugin;
import { wgs84togcj02, gcj02towgs84 } from './gps';

class NeTMap {
  constructor(dom, opt) {
    const define = {
      zoom: 17,
      center: new qq.maps.LatLng(39.90923, 116.397428),
      toolbar: false,
    };
    const center = opt.center;
    if (center) {
      opt.center = new qq.maps.LatLng(center[1], center[0]);
    }
    const option = Object.assign({}, define, opt);
    this.map = new qq.maps.Map(dom, option);
    if (option.toolbar) {
      new qq.maps.ScaleControl({
        map: this.map,
      });
    }

    this.markersArray = [];
  }

  setMap(opt) {
    const define = {
      zoom: this.map.getZoom(),
      center: this.map.getCenter(),
    };
    const center = opt.center;
    if (center) {
      opt.center = new qq.maps.LatLng(center[1], center[0]);
    }
    const option = Object.assign(define, opt);
    this.map.zoomTo(option.zoom);
    this.map.panTo(option.center);
  }

  addMarker(pos, opt, cb) {
    const define = {
      image: 'https://webapi.amap.com/theme/v1.3/markers/n/mark_b.png',
      size: new qq.maps.Size(19, 33),
    };
    const size = opt.size;
    if (size) {
      opt.size = new qq.maps.Size(size[0], size[1]);
    }
    if (opt.anchor) {
      opt.anchor = new qq.maps.Point(opt.anchor[0], opt.anchor[1]);
    } else {
      opt.anchor = new qq.maps.Point(size[0] / 2, size[1]);
    }
    const option = Object.assign({}, define, opt);
    const markerObj = {
      map: this.map,
      position: new qq.maps.LatLng(pos[1], pos[0]),
    };
    if (option.rotation) {
      markerObj.rotation = option.rotation;
    }
    const marker = new qq.maps.Marker(markerObj);
    const icon = new qq.maps.MarkerImage(
      option.image,
      option.size,
      new qq.maps.Point(0, 0),
      opt.anchor
    );
    marker.setIcon(icon);
    if (cb) {
      qq.maps.event.addListener(marker, 'click', cb);
    }
    this.markersArray.push(marker);
    return marker;
  }

  markerMove(marker, target, moveSpeed) {
    marker.moveTo(new qq.maps.LatLng(target[1], target[0]), moveSpeed);
  }

  clearMap() {
    const markersArray = this.markersArray;
    for (const i in markersArray) {
      markersArray[i].setMap(null);
    }
    this.heatmap && this.heatmap.setMap(null);
    this.heatmap = null;
  }

  openInfoWindow(pos, info) {
    const define = {
      map: this.map,
    };
    const infoWindow = new qq.maps.InfoWindow(define);
    infoWindow.open();
    infoWindow.setContent(info);
    infoWindow.setPosition(new qq.maps.LatLng(pos[1], pos[0]));
    return infoWindow;
  }

  addHeatMap(points, opt) {
    const define = {
      radius: 25,
      opacity: [ 0, 0.8 ],
      gradient: {
        0.4: 'blue',
        0.5: 'rgb(117,211,248)',
        0.6: 'rgb(0, 255, 0)',
        0.7: '#ffea00',
        0.8: 'red',
      },
    };
    const option = Object.assign({}, define, opt);
    const heatmap = new QQMapPlugin.HeatmapOverlay(this.map, {
      radius: 1,
      maxOpacity: option.opacity[1],
      useLocalExtrema: true,
      valueField: 'count',
    });
    console.log(heatmap);
    this.heatmap = heatmap;
    heatmap.setData({ data: points, max: 100 });
  }

  showHeatMap() {
    this.heatmap && this.heatmap.show();
  }

  hideHeatMap() {
    this.heatmap && this.heatmap.hide();
  }

  addMoveLine(lineArr, moveSpeed, lineOpt, markerOpt) {
    const pathArr = [];
    for (const item of lineArr) {
      pathArr.push(new qq.maps.LatLng(item[1], item[0]));
    }
    this.lineArr = pathArr;
    this.moveSpeed = moveSpeed;
    const defineLine = {
      map: this.map,
      path: pathArr,
      strokeColor: '#00A',
      strokeWeight: 3,
    };
    const lineOption = Object.assign({}, defineLine, lineOpt);
    new qq.maps.Polyline(lineOption);
    const defineMarker = {
      image: 'https://webapi.amap.com/images/car.png',
      size: new qq.maps.Size(32, 36),
    };
    const size = markerOpt.size;
    if (size) {
      markerOpt.size = new qq.maps.Size(size[0], size[1]);
    }
    const markerOption = Object.assign({}, defineMarker, markerOpt);
    const marker = new qq.maps.Marker({
      map: this.map,
      position: pathArr[0],
      autoRotation: true,
    });
    const icon = new qq.maps.MarkerImage(
      markerOption.image,
      markerOption.size,
      new qq.maps.Point(0, 0),
      new qq.maps.Point(size[0] / 2, size[1] / 2)
    );
    marker.setIcon(icon);
    this.marker = marker;
  }

  startMove() {
    this.marker.moveAlong(this.lineArr, this.moveSpeed);
  }

  pauseMove() {
    this.marker.pauseMove();
  }

  resumeMove() {
    this.marker.resumeMove();
  }

  stopMove() {
    this.marker.stopMove();
  }

  static convert(gps) {
    const latlng = [];
    for (const item of gps) {
      latlng.push(new qq.maps.LatLng(item[1], item[0]));
    }
    return new Promise((resolve, reject) => {
      qq.maps.convertor.translate(latlng, '3', result => {
        if (result) {
          const location = [];
          for (const item of result) {
            location.push([ item.lng, item.lat ]);
          }
          resolve(location);
        } else {
          reject(result);
        }
      });
    });
  }
  translateFromGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      const trans = wgs84togcj02(data[j].latitude, data[j].longitude);
      transRes.push(new qq.maps.LatLng(trans[0], trans[1]));
    }
    return transRes;
  }
  translateToGPS(data) {
    const transRes = [];
    for (let j = 0; j < data.length; j++) {
      let trans = [];
      if (data[j].lat) {
        trans = gcj02towgs84(data[j].lat, data[j].lng);
      } else {
        trans = gcj02towgs84(data[j].latitude, data[j].longitude);
      }
      transRes.push({ latitude: trans[0], longitude: trans[1] });
    }
    return transRes;
  }
  static geocode(location) {
    const coord = new qq.maps.LatLng(location[1], location[0]);
    return new Promise((resolve, reject) => {
      new qq.maps.Geocoder({
        complete: result => {
          if (result) {
            resolve(result.detail.address);
          } else {
            reject(result);
          }
        },
      }).getAddress(coord);
    });
  }
}

export default NeTMap;
