// 定义一些常量
const PI = 3.1415926535897932384626;
const a = 6378245.0;
const ee = 0.00669342162296594323;
const x_pi = 3.14159265358979324 * 3000.0 / 180.0;

// 世界大地坐标转为百度坐标
function wgs84tobd(lat, lng) {
  const wgs2gcjR = wgs84togcj02(lat, lng);
  const gcj2bdR = gcj02tobd(wgs2gcjR[0], wgs2gcjR[1]);
  return gcj2bdR;
}

function bdtowgs84(lat, lng) {
  const bd2gcjR = bdtogcj02(lat, lng);
  const gcjR2wgsR = gcj02towgs84(bd2gcjR[0], bd2gcjR[1]);
  return gcjR2wgsR;
}

function bdtogcj02(lat, lng) {
  const x_pi = 3.14159265358979324 * 3000.0 / 180.0;
  const x = lng - 0.0065;
  const y = lat - 0.006;
  const z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_pi);
  const theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_pi);
  const gg_lng = z * Math.cos(theta);
  const gg_lat = z * Math.sin(theta);
  return [ gg_lat, gg_lng ];
}

function gcj02tobd(lat, lng) {
  const x = lng;
  const y = lat;
  const z = Math.sqrt(x * x + y * y) + 0.00002 * Math.sin(y * x_pi);
  const theta = Math.atan2(y, x) + 0.000003 * Math.cos(x * x_pi);
  const bd_lng = z * Math.cos(theta) + 0.0065;
  const bd_lat = z * Math.sin(theta) + 0.006;
  const result = [];
  result.push(bd_lat);
  result.push(bd_lng);
  return result;
}

function wgs84togcj02(lat, lng) {
  if (out_of_china(lat, lng)) {
    return [ lat, lng ];
  }
  let dlat = transformlat(lng - 105.0, lat - 35.0);
  let dlng = transformlng(lng - 105.0, lat - 35.0);
  const radlat = lat / 180.0 * PI;
  let magic = Math.sin(radlat);
  magic = 1 - ee * magic * magic;
  const sqrtmagic = Math.sqrt(magic);
  dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * PI);
  dlng = (dlng * 180.0) / (a / sqrtmagic * Math.cos(radlat) * PI);
  const mglat = lat + dlat;
  const mglng = lng + dlng;
  return [ mglat, mglng ];
}

function gcj02towgs84(lat, lng) {
  let res = [];
  if (out_of_china(lat, lng)) {
    res = [ lat, lng ];
  } else {
    let dlat = transformlat(lng - 105.0, lat - 35.0);
    let dlng = transformlng(lng - 105.0, lat - 35.0);
    const radlat = lat / 180.0 * PI;
    let magic = Math.sin(radlat);
    magic = 1 - ee * magic * magic;
    const sqrtmagic = Math.sqrt(magic);
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * PI);
    dlng = (dlng * 180.0) / (a / sqrtmagic * Math.cos(radlat) * PI);
    const mglat = lat + dlat;
    const mglng = lng + dlng;
    res = [ lat * 2 - mglat, lng * 2 - mglng ];
  }
  return res;
}

function transformlat(lat, lng) {
  let ret = -100.0 + 2.0 * lat + 3.0 * lng + 0.2 * lng * lng + 0.1 * lat * lng + 0.2 * Math.sqrt(Math.abs(lat));
  ret += (20.0 * Math.sin(6.0 * lat * PI) + 20.0 * Math.sin(2.0 * lat * PI)) * 2.0 / 3.0;
  ret += (20.0 * Math.sin(lng * PI) + 40.0 * Math.sin(lng / 3.0 * PI)) * 2.0 / 3.0;
  ret += (160.0 * Math.sin(lng / 12.0 * PI) + 320 * Math.sin(lng * PI / 30.0)) * 2.0 / 3.0;
  return ret;
}

function transformlng(lat, lng) {
  let ret = 300.0 + lat + 2.0 * lng + 0.1 * lat * lat + 0.1 * lat * lng + 0.1 * Math.sqrt(Math.abs(lat));
  ret += (20.0 * Math.sin(6.0 * lat * PI) + 20.0 * Math.sin(2.0 * lat * PI)) * 2.0 / 3.0;
  ret += (20.0 * Math.sin(lat * PI) + 40.0 * Math.sin(lat / 3.0 * PI)) * 2.0 / 3.0;
  ret += (150.0 * Math.sin(lat / 12.0 * PI) + 300.0 * Math.sin(lat / 30.0 * PI)) * 2.0 / 3.0;
  return ret;
}

function out_of_china(lat, lng) {
  return (lng < 72.004 || lng > 137.8347) || ((lat < 0.8293 || lat > 55.8271) || false);
}

export {
  wgs84togcj02,
  gcj02towgs84,
  wgs84tobd,
  bdtowgs84,
};
